function [new_x_k_1, new_x_k_2, there_are_2,...
             is_different, is_empty, best_precond_no]...
    = composite_best_width_reduction(k, A, b, x, L)
% RBK, December 14, 2005 -- 
%  [new_x_k_1, new_x_k_2, there_are_2,... 
%   is_different, is_empty, precond_no]...
%    = composite_best_width_reduction(k, A, b, x)
% implements Algorithm 1 in
% R. B. Kearfott and S. Hongthong, "On preconditioners and
% splitting in the interval Gauss--Seidel method," 2005.
% for finding narrower bounds on the k-th coordinate of the
% portion of the solution of A z = b that lies in the box x.
% L is the number of subdivisions in delta to be used for the
% mignitude-optimal preconditioner.
% On return, precond_no is set to:
% 1 if the width-optimal preconditioner led to success
% 2 if the negative numerator S-preconditioner led to success
% 3 if the positive numerator S-preconditioner led to success
% 4+i if the mignitude-optimal preconditioner led to success,
%   with delta = i(1/L).
% -1 if is-different is false (that is, if there was no
%   success).
% Based on the seven examples (ibid.), we use linprog
% for the width-optimal preconditioner and quadprog for
% the other preconditioners.

% The penalty factor for the quadratic formulation -- 
rho = 1000;

new_x_k_1 = x(k);
new_x_k_2 = infsup(Inf,Inf);
there_are_2 = 0;
is_different = 0;
is_empty = 0;

% Step 2: compute with the width-optimal preconditioner --
best_precond_no = -1;
precond_no = 1;
delta = rand(1);
[f, Aineq, bineq, Aeq, beq, lb, H] = ...
                C_LP_def(delta, x, A, b, k, rho);
[ypm,objmin,exitflag,output,lambda] = ...
            linprog(f,Aineq,bineq,Aeq,beq,lb);
Y_k = extract_y(A,ypm);
[new_x_k_w, second_new_x_k_w, there_are_2_w, num, denom] ...
        = gauss_seidel_step(k, Y_k, A, b, x);
[xi1, xi2, there_are_2_w,is_empty, is_different]... 
    = extended_intersect ...
    (new_x_k_1, new_x_k_w, second_new_x_k_w, there_are_2_w);
if (is_different) 
    best_precond_no = precond_no;
end
new_x_k_1 = xi1;
there_are_2 = there_are_2_w;
if(there_are_2)
    new_x_k_2 = xi2;
end
if(is_empty)
    return
end

% Step 5: Compute with the negative numerator S-preconditioner --
precond_no = 2;
delta = rand(1);
[f, Aineq, bineq, Aeq, beq, lb, H] = ...
                neg_num_S_LP_def(delta, x, A, b, k, rho);
[ypm,objmin,exitflag,output,lambda] = ...
            quadprog_buffer(H,f,Aineq,bineq,Aeq,beq,lb);
Y_k = extract_y(A,ypm);
[new_x_k_s, second_new_x_k_s, there_are_2_s, num, denom] ...
        = gauss_seidel_step(k, Y_k, A, b, x);
[xi1, xi2, there_are_2,is_empty, is_different_s]... 
       = extended_intersect_with_2 ...
    (new_x_k_1, new_x_k_2, there_are_2,...
     new_x_k_s, second_new_x_k_s, there_are_2_s);
if (is_different_s) 
    best_precond_no = precond_no;
end
is_different = is_different | is_different_s
new_x_k_1 = xi1;
if(there_are_2)
    new_x_k_2 = xi2;
end
if(is_empty)
    return
end

% Step 8: Compute with the positive numerator S-preconditioner --
precond_no = 3;
delta = rand(1);
[f, Aineq, bineq, Aeq, beq, lb, H] = ...
                pos_num_S_LP_def(delta, x, A, b, k, rho);
[ypm,objmin,exitflag,output,lambda] = ...
            quadprog_buffer(H,f,Aineq,bineq,Aeq,beq,lb);
Y_k = extract_y(A,ypm);
[new_x_k_s, second_new_x_k_s, there_are_2_s, num, denom] ...
        = gauss_seidel_step(k, Y_k, A, b, x);
[xi1, xi2, there_are_2,is_empty, is_different_s]... 
       = extended_intersect_with_2 ...
    (new_x_k_1, new_x_k_2, there_are_2,...
     new_x_k_s, second_new_x_k_s, there_are_2_s);
if (is_different_s) 
    best_precond_no = precond_no;
end
is_different = is_different | is_different_s
new_x_k_1 = xi1;
if(there_are_2)
    new_x_k_2 = xi2;
end
if(is_empty)
    return
end

% Steps 11 and 12: Try the mignitude-optimal preconditioners --
h = 1/L;
for i=0:L
    precond_no = 4+i;
    delta = i*h;
    [f, Aineq, bineq, Aeq, beq, lb, H] = ...
                mig_opt_LP_def(delta, x, A, b, k, rho);
    [ypm,objmin,exitflag,output,lambda] = ...
            quadprog_buffer(H,f,Aineq,bineq,Aeq,beq,lb);
    Y_k = extract_y(A,ypm);
    [new_x_k_s, second_new_x_k_s, there_are_2_s, num, denom] ...
        = gauss_seidel_step(k, Y_k, A, b, x);
    [xi1, xi2, there_are_2,is_empty, is_different_s]... 
           = extended_intersect_with_2 ...
        (new_x_k_1, new_x_k_2, there_are_2,...
         new_x_k_s, second_new_x_k_s, there_are_2_s);
    if (is_different_s) 
        best_precond_no = precond_no;
    end
    is_different = is_different | is_different_s
    new_x_k_1 = xi1;
    if(there_are_2)
        new_x_k_2 = xi2;
    end
    if(is_empty)
        return
    end
end
