function [f, Aineq, bineq, Aeq, beq, lb, H] = ...
    mig_opt_LP_def(delta, x, A, b, k, rho)
% [f, Aeq, beq, lb, H] = mig_opt_LP_def(delta, x, A, b, k, rho) 
% sets up theobjective f, constraint matrix Aeq, and lower bound 
% vector lbfor the C-LP preconditioner as in 
% [2005-new-S-preconditioner]
% for use in the optimization toolbox function linprog.
% The input arguments A and b do not correspond to inequality
% constraints for linprog, but are an interval matrix and
% interval vector for the m by n interval system A x = b
% for which the preconditioner is being computed.
% for linprog, but is the
% The variables are ordered as 
%   (y_1^+, y_2^+,..., y_m^+, y_1^-, y_2^-,..., y_m^-,
%    v_1^+, ..., v_{i-1}^+, v_{i+1}^+, ..., v_n^+, 
%    v_1^-, ..., v_{i-1}^-, v_{i+1}^-, ..., v_n^-) 
% On return, the matrix H contains the quadratic form for
% the penalty term rho y^{+}^T y^{-}, if it is desired
% to use the optimization toolbox routine quadprog.
% The input argument epsilon enters into constraints
% that force \Inf{denominator} \le -epsilon and
%            \Sup(denominator) \ge  epsilon.

n = size(x,1);
m = size(A,1);
nvar = 2*(m+(n-1)+1);
nconstr = (n-1) + 1;
ypstart = 0;
ymstart = ypstart+m;
vpstart = ymstart+m;
vmstart = vpstart+n;
if (nargin >= 6)
    H = zeros(nvar,nvar);
    for i=1:m
        H(ypstart+i,ymstart+i)=rho;
        H(ymstart+i,ypstart+i)=rho;
    end
    for j = 1:n;
        H(vpstart+j,vmstart+j) = rho;
        H(vmstart+j,vpstart+j) = rho;
    end
end

Aineq = [];
bineq = [];
% Simultaneously fill the objective and constraints to
% avoid multiple nested loops --
f = zeros(nvar,1);
Aeq = zeros(nconstr,nvar);
beq = zeros(nconstr,1);
beq(1) = 1;
lb = zeros(nvar,1);

% To have consistent numbering with the C-W preconditioner,
% v_k is actually stored in v_n
f(vpstart+n) = delta;
f(vmstart+n) = 1-delta;
for i=1:m;
    f(ypstart+i) = -delta*inf(A(i,k)) + (1-delta)*sup(A(i,k));
    f(ymstart+i) =  delta*sup(A(i,k)) - (1-delta)*inf(A(i,k));
    Aeq(1,ypstart+i) =  mid(b(i)) - rad(b(i));
    Aeq(1,ymstart+i) = -mid(b(i)) - rad(b(i));
    Aeq(n,ypstart+i) = -(inf(A(i,k))+sup(A(i,k)));
    Aeq(n,ymstart+i) =  (inf(A(i,k))+sup(A(i,k)));
    Aeq(n,vpstart+n) =  1;
    Aeq(n,vmstart+n) = -1;
    vcounter = 0;
   % The interiors of the following two loops should be identical.
    for j=1:k-1;
        fill_inner_part
    end
    for j=k+1:n;
        fill_inner_part
    end
    
end

    function fill_inner_part
        vcounter = vcounter + 1;
        Aeq(1,ypstart+i) = Aeq(1,ypstart+i) + 2*rad(x(j)) *...
            ((delta/2)*inf(A(i,j)) - ((1-delta)/2)*sup(A(i,j)));
        Aeq(1,ymstart+i) = Aeq(1,ymstart+i) + 2*rad(x(j)) *...
            (-(delta/2)*sup(A(i,j)) + ((1-delta)/2)*inf(A(i,j)));
        Aeq(1,vpstart+vcounter) = -(delta/2)*2*rad(x(j));
        Aeq(1,vmstart+vcounter) = -((1-delta)/2)*2*rad(x(j));
        Aeq(1+vcounter,ypstart+i) = -(inf(A(i,j))+sup(A(i,j)));
        Aeq(1+vcounter,ymstart+i) =  (inf(A(i,j))+sup(A(i,j)));
        Aeq(1+vcounter,vpstart+vcounter) =  1;
        Aeq(1+vcounter,vmstart+vcounter) = -1;
    end
end
