function run_preconditioner_test(A, b, x, k,... 
    pre_no, quadprog, filename)
% RBK 11/30/2005 -- This routine computes preconditioners for
% the k-th coordinate, then performs Gauss--Seidel iteration.
% It outputs the results to the file specified in the character
% string filename.  The system to be solved is A x = b, while
% the input interval argument x contains bounds within which
% this system is to be solved.  The argument pre_no is a flag
% that specifies the preconditioner type, as follows:
%  pre_no = 1: width-optimal preconditioner
%  pre_no = 2: mignitude-optimal preconditioner
%  pre_no = 3: negative numerator S preconditioner
%  pre_no = 4: positive numerator S preconditioner
%  pre_no = 5: inverse-midpoint preconditioner
%  pre_no = 6: no preconditioner, run through all possibilities
% quadprog is a binary variable, such that
%  quadprog = 0: The linear programming solver linprog is
%                used to compute the preconditioner.
%  quadprog = 1: The quadratic programming solver quadprog is
%                used to compute the preconditioner; 
%                \rho = 1000 is used for the quadratic
%                penalty function.
%
%  The preconditioners and Gauss--Seidel steps are computed
%  At 11 delta values: delta = 0, .1, .2, ..., .9, 1.
%
fid = fopen(filename, 'w');
timevec = clock;
fprintf(fid, 'Output from run_preconditioner_test on ');
fprintf(fid, '%4.0f/%2.0f/%2.0f at %2.0f:%2.0f.%2.0f.\n',...
    timevec(1:6)');
fprintf(fid,'filename:');
fprintf(fid,filename);
fprintf(fid,'\n\n');
m = size(A,1);
n = size(A,2);
for i=1:m;
    for j=1:n;
        fprintf(fid, 'A(%2.0f,%2.0f):' , i, j);
        fprintf(fid, ' [%12.4e, %12.4e]\n',inf(A(i,j)),sup(A(i,j)));
    end
end
fprintf(fid,'\n');
for i=1:m;
    fprintf(fid, 'b(%2.0f): ', i);
    fprintf(fid, '[%12.4e, %12.4e]\n', inf(b(i)), sup(b(i)));
end
fprintf(fid,'\n');
for i=1:m;
    fprintf(fid, 'x(%2.0f): ', i);
    fprintf(fid, '[%12.4e, %12.4e]\n', inf(x(i)), sup(x(i)));
end
fprintf(fid,'\n');
fprintf(fid, 'k = %2.0f, ', k);
fprintf(fid, 'pre_no = %1.0f, ', pre_no);

fprintf(fid, 'quadprog = %1.0f. ', quadprog);
fprintf(fid,'\n');
N = 10;
h = 1/N;
rho = 1e3;
for i=0:N;
    delta = i*h;
    switch (pre_no)
        case(1)
            [f, Aineq, bineq, Aeq, beq, lb, H] = ...
                C_LP_def(delta, x, A, b, k, rho);
        case(2)
            [f, Aineq, bineq, Aeq, beq, lb, H] = ...
                mig_opt_LP_def(delta, x, A, b, k, rho);
        case(3)
            [f, Aineq, bineq, Aeq, beq, lb, H] = ...
                neg_num_S_LP_def(delta, x, A, b, k, rho);
        case(4)
            [f, Aineq, bineq, Aeq, beq, lb, H] = ...
                pos_num_S_LP_def(delta, x, A, b, k, rho);
        case(5)
            Y = inv(imid(A));
            Y_k = Y(k,:);
        case(6)
            Y_k = zeros(1,n);
            Y_k(k) = 1;
        otherwise
            fprintf(fid,'Unknown preconditioner type: %2.0f\n', ...
                pre_no);
            fprintf('Unknown preconditioner type: %2.0f\n', ...
                pre_no);
            return
            fclose(fid);
    end
    if(pre_no~=5 & pre_no~=6)
        if (quadprog)
            [ypm,objmin,exitflag,output,lambda] = ...
                quadprog_buffer(H,f,Aineq,bineq,Aeq,beq,lb);
        else
            [ypm,objmin,exitflag,output,lambda] = ...
                linprog(f,Aineq,bineq,Aeq,beq,lb);
        end
        Y_k = extract_y(A,ypm);
    end
    
    if(pre_no~=6)
        fprintf(fid,'\n');
        fprintf(fid,'delta = %2.1f, ',delta);
        fprintf(fid,'exitflag = %2.0f, and objmin = %12.4e\n',...
            exitflag, objmin);
        fprintf(fid,'ypm:\n');
        fprintf(fid, '%12.4e, %12.4e, %12.4e, %12.4e, %12.4e\n',ypm);
    end
    fprintf('\n');
    fprintf(fid,'Y_k:\n');
    fprintf(fid, '%12.4e, %12.4e, %12.4e, %12.4e, %12.4e\n',Y_k);
    fprintf(fid,'\n');
    
    if(pre_no ~=6)
        [new_x_k, second_new_x_k, there_are_2, num, denom] = ...
            gauss_seidel_step(k, Y_k, A, b, x);
    else
        [new_x, second_new_x, there_are_2, ...
            is_different, is_empty] = ...
            all_coordinates_no_preconditioner(k, A, b, x);
        new_x_k = new_x(k);
        second_new_x_k = second_new_x(k);
    end
    
    if(pre_no ~= 6)
        fprintf(fid, '  numerator: [%12.4f, %12.4f]\n', ...
            inf(num), sup(num));
        fprintf(fid, 'denominator: [%12.4f, %12.4f]\n', ...
            inf(denom), sup(denom));
    else
        fprintf(fid, '   is_different, is_empty: [%12.4f, %12.4f]\n',... 
            is_different, is_empty);
    end
    fprintf(fid, 'there_are_2: %1.0f\n', there_are_2);
    fprintf(fid, 'new_x_k:  [%12.4f, %12.4f]\n',...
        inf(new_x_k), sup(new_x_k));
    if(there_are_2)
        fprintf(fid, 'second_new_x_k:  [%12.4f, %12.4f]\n',...
            inf(second_new_x_k), sup(second_new_x_k));
    end
end
fclose(fid);