function [new_x] = solution_component_in_orthant...
    (A, b, x, orthant)
% [new_x] = solution_component_in_orthant(A, b, x, orthant)
%   returns bounds on that portion of the solution of A x = b
%   within x that lie in the orthant defined by the vector
%   orthant.  (orthant is a vector that contains a 1 in the
%   i-th position if the i-th variable is non-negative, and
%   a -1 in the i-th position otherwise.
%   It uses the procedure outlined by Rohn on p. 172, 
%   "Reliability in Computing: The Role of Interval Methods
%   in Scientific Computing," ed. R. E. Moore, Academic Press,
%   1988 (originally from Oettli, SINUM, 1965).
%   This is presently non-rigorous, since the LP outputs
%   are not validated.
% RBK 11/19/2005

m = size(A,1);
n = size(A,2);

% The matrices and vectors in the inequalities below
% theorem 1.1, p. 172 of Rohn (1988) --
bpos = sup(b);
bneg = inf(b);
Tz = diag(orthant);
Amid = mid(A);
Delta = rad(A);

Aneg = Amid - Delta*Tz;
Apos = Amid + Delta*Tz;

%Fill the inequality constraints linprog --
lb = inf(x);
ub = sup(x);
Aineq = zeros(2*m+n,n);
bineq = zeros(2*m+n,1);

Aineq(1:m,1:n) = Aneg;
bineq(1:m) = bpos;
Aineq(m+1:2*m,1:n) = -Apos;
bineq(m+1:2*m) = -bneg;
Aineq(2*m+1:2*m+n,1:n) = -Tz;


% Set up the objective and solve the linear programs
% for each bound on each coordinate --
f = zeros(n,1);
for i = 1:n
    success = 1;
    f(i) = 1;
    [lppoint,inf_x_i,exit] = linprog(f,Aineq,bineq,[],[],lb,ub);
    if (exit ~=1) 
        success = 0;
    end
    f(i)=-1;
    [lppoint,sup_x_i,exit] = linprog(f,Aineq,bineq,[],[],lb,ub);
    if (exit ~=1) 
        success = 0;
    end
    sup_x_i = -sup_x_i;
    if (success)
        if inf_x_i <= sup_x_i
            new_x(i) = infsup(inf_x_i,sup_x_i);
        else
            new_x(i) = infsup(sup_x_i,inf_x_i);
        end
    else
        new_x(i) = x(i);
    end
    f(i) = 0;
end



